@@ -114,7 +114,7 @@ gst_identity_class_init (GstIdentityClass *klass)
                         0.0, 1.0, 0.0, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SILENT,
     g_param_spec_boolean ("silent", "silent", "silent",
-                          TRUE,G_PARAM_READWRITE)); 
+                          FALSE, G_PARAM_READWRITE)); 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_LAST_MESSAGE,
     g_param_spec_string ("last-message", "last-message", "last-message",
                          NULL, G_PARAM_READABLE)); 
@@ -154,8 +154,8 @@ gst_identity_getcaps (GstPad *pad, GstCaps *caps)
   return gst_pad_get_allowed_caps (otherpad);
 }
 
-static GstPadConnectReturn
-gst_identity_connect (GstPad *pad, GstCaps *caps)
+static GstPadLinkReturn
+gst_identity_link (GstPad *pad, GstCaps *caps)
 {
   GstIdentity *identity;
   GstPad *otherpad;
@@ -166,7 +166,7 @@ gst_identity_connect (GstPad *pad, GstCaps *caps)
   if (GST_CAPS_IS_FIXED (caps))
     return gst_pad_try_set_caps (otherpad, caps);
   else
-    return GST_PAD_CONNECT_DELAYED;
+    return GST_PAD_LINK_DELAYED;
 }
 
 static void 
@@ -176,12 +176,12 @@ gst_identity_init (GstIdentity *identity)
   gst_element_add_pad (GST_ELEMENT (identity), identity->sinkpad);
   gst_pad_set_chain_function (identity->sinkpad, GST_DEBUG_FUNCPTR (gst_identity_chain));
   gst_pad_set_bufferpool_function (identity->sinkpad, gst_identity_get_bufferpool);
-  gst_pad_set_connect_function (identity->sinkpad, gst_identity_connect);
+  gst_pad_set_link_function (identity->sinkpad, gst_identity_link);
   gst_pad_set_getcaps_function (identity->sinkpad, gst_identity_getcaps);
   
   identity->srcpad = gst_pad_new ("src", GST_PAD_SRC);
   gst_element_add_pad (GST_ELEMENT (identity), identity->srcpad);
-  gst_pad_set_connect_function (identity->srcpad, gst_identity_connect);
+  gst_pad_set_link_function (identity->srcpad, gst_identity_link);
   gst_pad_set_getcaps_function (identity->srcpad, gst_identity_getcaps);
 
   identity->loop_based = FALSE;
@@ -220,7 +220,8 @@ gst_identity_chain (GstPad *pad, GstBuffer *buf)
       if (identity->last_message != NULL) {
 	g_free (identity->last_message);
       }
-      identity->last_message = g_strdup_printf ("dropping   ******* (%s:%s)i (%d bytes, %llu)",
+      identity->last_message = g_strdup_printf ("dropping   ******* (%s:%s)i (%d bytes, %"
+                                                G_GINT64_FORMAT ")",
 	      GST_DEBUG_PAD_NAME (identity->sinkpad), GST_BUFFER_SIZE (buf), GST_BUFFER_TIMESTAMP (buf));
       g_object_notify (G_OBJECT (identity), "last-message");
       gst_buffer_unref (buf);
@@ -232,13 +233,14 @@ gst_identity_chain (GstPad *pad, GstBuffer *buf)
   }
 
   for (i = identity->duplicate; i; i--) {
-    if (!identity->silent)
-      if (identity->last_message != NULL) {
-	g_free (identity->last_message);
-      }
-      identity->last_message = g_strdup_printf ("chain   ******* (%s:%s)i (%d bytes, %llu)",
+    if (!identity->silent) {
+      g_free (identity->last_message);
+      identity->last_message = g_strdup_printf ("chain   ******* (%s:%s)i (%d bytes, %"
+                                                G_GINT64_FORMAT ")",
 	      GST_DEBUG_PAD_NAME (identity->sinkpad), GST_BUFFER_SIZE (buf), GST_BUFFER_TIMESTAMP (buf));
-    g_object_notify (G_OBJECT (identity), "last-message");
+      g_object_notify (G_OBJECT (identity), "last-message");
+    }
+
     g_signal_emit (G_OBJECT (identity), gst_identity_signals[SIGNAL_HANDOFF], 0,
 	                       buf);
 
@@ -265,10 +267,18 @@ gst_identity_loop (GstElement *element)
   
   buf = gst_pad_pull (identity->sinkpad);
   if (GST_IS_EVENT (buf)) {
-    gst_pad_event_default (identity->sinkpad, GST_EVENT (buf));
-  }
+    GstEvent *event = GST_EVENT (buf);
 
-  gst_identity_chain (identity->sinkpad, buf);
+    if (GST_EVENT_IS_INTERRUPT (event)) {
+      gst_event_unref (event);
+    }
+    else {
+      gst_pad_event_default (identity->sinkpad, event);
+    }
+  }
+  else {
+    gst_identity_chain (identity->sinkpad, buf);
+  }
 }
 
 static void 
