@@ -166,8 +166,10 @@ gst_init_check (int *argc, char **argv[])
 void
 gst_init (int *argc, char **argv[])
 {
-  if (!gst_init_with_popt_table (argc, argv, NULL))
-    g_error ("Could not initialize GStreamer !\n");
+  if (!gst_init_with_popt_table (argc, argv, NULL)) {
+    g_print ("Could not initialize GStreamer !\n");
+    exit (1);
+  }
 }
 
 /**
@@ -392,8 +394,15 @@ init_pre (void)
   gst_registry_add_path (_global_registry, PLUGINS_DIR);
 #endif /* PLUGINS_USE_BUILDDIR */
 
-  homedir = g_get_home_dir ();
-  user_reg = g_strjoin ("/", homedir, LOCAL_REGISTRY_FILE, NULL);
+  if (g_getenv ("GST_REGISTRY"))
+  {
+    user_reg = g_strdup (g_getenv ("GST_REGISTRY"));
+  }
+  else
+  {
+    homedir = g_get_home_dir ();
+    user_reg = g_strjoin ("/", homedir, LOCAL_REGISTRY_FILE, NULL);
+  }
   _user_registry = gst_xml_registry_new ("user_registry", user_reg);
 
 #ifndef GST_DISABLE_REGISTRY
