@@ -25,10 +25,10 @@
 
 #include "gstcaps.h"
 #include "gsttype.h"
+#include "gstmemchunk.h"
 #include "gstlog.h"
 
-static GMemChunk *_gst_caps_chunk;
-static GMutex *_gst_caps_chunk_lock;
+static GstMemChunk *_gst_caps_chunk;
 
 GType _gst_caps_type;
 
@@ -52,6 +52,7 @@ transform_func (const GValue *src_value,
     g_value_set_boxed  (&value, caps->properties);
     props = g_strdup_value_contents (&value);
 
+    g_value_unset (&value);
     g_string_append (result, props);
     g_free (props);
 
@@ -65,10 +66,9 @@ transform_func (const GValue *src_value,
 void
 _gst_caps_initialize (void)
 {
-  _gst_caps_chunk = g_mem_chunk_new ("GstCaps",
+  _gst_caps_chunk = gst_mem_chunk_new ("GstCaps",
                   sizeof (GstCaps), sizeof (GstCaps) * 256,
                   G_ALLOC_AND_FREE);
-  _gst_caps_chunk_lock = g_mutex_new ();
 
   _gst_caps_type = g_boxed_type_register_static ("GstCaps",
                                        (GBoxedCopyFunc) gst_caps_ref,
@@ -77,6 +77,13 @@ _gst_caps_initialize (void)
   g_value_register_transform_func (_gst_caps_type,
 		                   G_TYPE_STRING,
 				   transform_func);
+
+}
+
+GType
+gst_caps_get_type (void)
+{
+  return _gst_caps_type;
 }
 
 static guint16
@@ -134,9 +141,12 @@ gst_caps_new_id (const gchar *name, const guint16 id, GstProps *props)
 {
   GstCaps *caps;
 
-  g_mutex_lock (_gst_caps_chunk_lock);
-  caps = g_mem_chunk_alloc (_gst_caps_chunk);
-  g_mutex_unlock (_gst_caps_chunk_lock);
+  caps = gst_mem_chunk_alloc (_gst_caps_chunk);
+
+  GST_DEBUG (GST_CAT_CAPS, "new %p", caps);
+
+  gst_props_ref (props);
+  gst_props_sink (props);
 
   caps->name = g_strdup (name);
   caps->id = id;
@@ -153,6 +163,14 @@ gst_caps_new_id (const gchar *name, const guint16 id, GstProps *props)
   return caps;
 }
 
+/**
+ * gst_caps_replace:
+ * @oldcaps: the caps to take replace
+ * @newcaps: the caps to take replace 
+ *
+ * Replace the pointer to the caps, doing proper
+ * refcounting.
+ */
 void
 gst_caps_replace (GstCaps **oldcaps, GstCaps *newcaps)
 {
@@ -195,11 +213,12 @@ gst_caps_destroy (GstCaps *caps)
 
   next = caps->next;
 
+  GST_DEBUG (GST_CAT_CAPS, "destroy %p", caps);
+
   gst_props_unref (caps->properties);
   g_free (caps->name);
-  g_mutex_lock (_gst_caps_chunk_lock);
-  g_mem_chunk_free (_gst_caps_chunk, caps);
-  g_mutex_unlock (_gst_caps_chunk_lock);
+
+  gst_mem_chunk_free (_gst_caps_chunk, caps);
 
   if (next) 
     gst_caps_unref (next);
@@ -217,8 +236,8 @@ gst_caps_debug (GstCaps *caps, const gchar *label)
 {
   GST_DEBUG_ENTER ("caps debug: %s", label);
   while (caps) {
-    GST_DEBUG (GST_CAT_CAPS, "caps: %p %s %s (%sfixed) (refcount %d) %s",
-               caps, caps->name, gst_caps_get_mime (caps),
+    GST_DEBUG (GST_CAT_CAPS, "caps: %p %s %s (%sfixed) (refcount %d) %s", 
+	       caps, caps->name, gst_caps_get_mime (caps), 
                GST_CAPS_IS_FIXED (caps) ? "" : "NOT ", caps->refcount,
                GST_CAPS_IS_FLOATING (caps) ? "FLOATING" : "");
 
@@ -253,6 +272,9 @@ gst_caps_unref (GstCaps *caps)
 
   g_return_val_if_fail (caps->refcount > 0, NULL);
 
+  GST_DEBUG (GST_CAT_CAPS, "unref %p (%d->%d) %d", 
+	     caps, caps->refcount, caps->refcount-1, GST_CAPS_FLAGS (caps));
+
   caps->refcount--;
   zero = (caps->refcount == 0);
 
@@ -274,7 +296,13 @@ gst_caps_unref (GstCaps *caps)
 GstCaps*
 gst_caps_ref (GstCaps *caps)
 {
-  g_return_val_if_fail (caps != NULL, NULL);
+  if (caps == NULL)
+    return NULL;
+
+  g_return_val_if_fail (caps->refcount > 0, NULL);
+
+  GST_DEBUG (GST_CAT_CAPS, "ref %p (%d->%d) %d", 
+	     caps, caps->refcount, caps->refcount+1, GST_CAPS_FLAGS (caps));
 
   caps->refcount++;
 
@@ -307,7 +335,7 @@ gst_caps_sink (GstCaps *caps)
  *
  * Copies the caps, not copying any chained caps.
  *
- * Returns: a copy of the GstCaps structure.
+ * Returns: a floating copy of the GstCaps structure.
  */
 GstCaps*
 gst_caps_copy_1 (GstCaps *caps)
@@ -331,7 +359,7 @@ gst_caps_copy_1 (GstCaps *caps)
  *
  * Copies the caps.
  *
- * Returns: a copy of the GstCaps structure.
+ * Returns: a floating copy of the GstCaps structure.
  */
 GstCaps*
 gst_caps_copy (GstCaps *caps)
@@ -362,7 +390,7 @@ gst_caps_copy (GstCaps *caps)
  * Copies the caps if the refcount is greater than 1
  *
  * Returns: a pointer to a GstCaps strcuture that can
- * be safely written to
+ * be safely written to.
  */
 GstCaps*
 gst_caps_copy_on_write (GstCaps *caps)
@@ -410,9 +438,7 @@ gst_caps_set_name (GstCaps *caps, const gchar *name)
 {
   g_return_if_fail (caps != NULL);
 
-  if (caps->name)
-    g_free (caps->name);
-
+  g_free (caps->name);
   caps->name = g_strdup (name);
 }
 
@@ -499,10 +525,13 @@ GstCaps*
 gst_caps_set_props (GstCaps *caps, GstProps *props)
 {
   g_return_val_if_fail (caps != NULL, caps);
-  g_return_val_if_fail (props != NULL, caps);
-  g_return_val_if_fail (caps->properties == NULL, caps);
 
-  caps->properties = props;
+  gst_props_replace_sink (&caps->properties, props);
+
+  if (props && !GST_PROPS_IS_FIXED (props))
+    GST_CAPS_FLAG_UNSET (caps, GST_CAPS_FIXED);
+  else
+    GST_CAPS_FLAG_SET (caps, GST_CAPS_FIXED);
 
   return caps;
 }
@@ -590,7 +619,7 @@ gst_caps_append (GstCaps *caps, GstCaps *capstoadd)
   while (caps->next) {
     caps = caps->next;
   }
-  caps->next = capstoadd;
+  gst_caps_replace_sink (&caps->next, capstoadd);
 
   return orig;
 }
@@ -617,7 +646,7 @@ gst_caps_prepend (GstCaps *caps, GstCaps *capstoadd)
   while (capstoadd->next) {
     capstoadd = capstoadd->next;
   }
-  capstoadd->next = caps;
+  gst_caps_replace_sink (&capstoadd->next, caps);
 
   return orig;
 }
@@ -746,6 +775,8 @@ gst_caps_intersect_func (GstCaps *caps1, GstCaps *caps2)
   props = gst_props_intersect (caps1->properties, caps2->properties);
   if (props) {
     result = gst_caps_new_id ("intersect", caps1->id, props);
+    gst_caps_ref (result);
+    gst_caps_sink (result);
   }
 
   return result;
@@ -759,7 +790,7 @@ gst_caps_intersect_func (GstCaps *caps1, GstCaps *caps2)
  * Make the intersection between two caps.
  *
  * Returns: The intersection of the two caps or NULL if the intersection
- * is empty.
+ * is empty. unref the caps after use.
  */
 GstCaps*
 gst_caps_intersect (GstCaps *caps1, GstCaps *caps2)
@@ -771,11 +802,16 @@ gst_caps_intersect (GstCaps *caps1, GstCaps *caps2)
 		  
   if (caps1 == NULL) {
     GST_DEBUG (GST_CAT_CAPS, "first caps is NULL, return other caps");
-    return gst_caps_copy (caps2);
+    return gst_caps_ref (caps2);
   }
   if (caps2 == NULL) {
     GST_DEBUG (GST_CAT_CAPS, "second caps is NULL, return other caps");
-    return gst_caps_copy (caps1);
+    return gst_caps_ref (caps1);
+  }
+
+  /* same caps */
+  if (caps1 == caps2) {
+    return gst_caps_ref (caps1);
   }
 
   while (caps1) {
@@ -796,7 +832,27 @@ gst_caps_intersect (GstCaps *caps1, GstCaps *caps2)
       }
       othercaps = othercaps->next;
     }
-    caps1 =  caps1->next;
+    caps1 = caps1->next;
+  }
+
+  return result;
+}
+
+GstCaps*
+gst_caps_union (GstCaps *caps1, GstCaps *caps2)
+{
+  GstCaps *result = NULL;
+
+  /* printing the name is not useful here since caps can be chained */
+  GST_DEBUG (GST_CAT_CAPS, "making union of caps %p and %p", caps1, caps2);
+		  
+  if (caps1 == NULL) {
+    GST_DEBUG (GST_CAT_CAPS, "first caps is NULL, return other caps");
+    return gst_caps_ref (caps2);
+  }
+  if (caps2 == NULL) {
+    GST_DEBUG (GST_CAT_CAPS, "second caps is NULL, return other caps");
+    return gst_caps_ref (caps1);
   }
 
   return result;
@@ -809,43 +865,40 @@ gst_caps_intersect (GstCaps *caps1, GstCaps *caps2)
  * Make the normalisation of the caps. This will return a new caps
  * that is equivalent to the input caps with the exception that all
  * lists are unrolled. This function is useful when you want to iterate
- * the caps.
+ * the caps. unref the caps after use.
  *
- * Returns: The normalisation of the caps.
+ * Returns: The normalisation of the caps. Unref after usage.
  */
 GstCaps*
 gst_caps_normalize (GstCaps *caps)
 {
-  GstCaps *result = NULL, *walk = caps;
+  GstCaps *result = NULL, *walk;
 
   if (caps == NULL)
     return caps;
 
+  GST_DEBUG (GST_CAT_CAPS, "normalizing caps %p ", caps);
+
+  walk = caps;
+
   while (caps) {
     GList *proplist;
 
     proplist = gst_props_normalize (caps->properties);
-    if (proplist && g_list_next (proplist) == NULL) {
-      if (result == NULL)
-	walk = result = caps;
-      else {
-	walk = walk->next = caps;
-      }
-      goto next;
-    }
-
     while (proplist) {
       GstProps *props = (GstProps *) proplist->data;
       GstCaps *newcaps = gst_caps_new_id (caps->name, caps->id, props);
 
+      gst_caps_ref (newcaps);
+      gst_caps_sink (newcaps);
+
       if (result == NULL)
 	walk = result = newcaps;
       else {
-	walk = walk->next = newcaps;
+ 	walk = walk->next = newcaps;
       }
       proplist = g_list_next (proplist);  
     }
-next:
     caps = caps->next;
   }
   return result;
@@ -905,9 +958,7 @@ gst_caps_load_thyself (xmlNodePtr parent)
       gchar *content;
       GstCapsFlags fixed = GST_CAPS_FIXED;
 
-      g_mutex_lock (_gst_caps_chunk_lock);
-      caps = g_mem_chunk_alloc0 (_gst_caps_chunk);
-      g_mutex_unlock (_gst_caps_chunk_lock);
+      caps = gst_mem_chunk_alloc0 (_gst_caps_chunk);
 
       caps->refcount = 1;
       GST_CAPS_FLAG_SET (caps, GST_CAPS_FLOATING);
@@ -923,7 +974,12 @@ gst_caps_load_thyself (xmlNodePtr parent)
           g_free (content);
         }
         else if (!strcmp (subfield->name, "properties")) {
-          caps->properties = gst_props_load_thyself (subfield);
+          GstProps *props = gst_props_load_thyself (subfield);
+
+	  gst_props_ref (props);
+	  gst_props_sink (props);
+          caps->properties = props;
+
           fixed &= (GST_PROPS_IS_FIXED (caps->properties) ? GST_CAPS_FIXED : 0 );
         }
 	
