@@ -44,7 +44,9 @@ typedef enum {
   GST_EVENT_SEGMENT_DONE,
   GST_EVENT_SIZE,
   GST_EVENT_RATE,
-  GST_EVENT_FILLER
+  GST_EVENT_FILLER,
+  GST_EVENT_TS_OFFSET,
+  GST_EVENT_INTERRUPT
 } GstEventType;
 
 extern GType _gst_event_type;
@@ -57,6 +59,8 @@ extern GType _gst_event_type;
 #define GST_EVENT_TIMESTAMP(event)	(GST_EVENT(event)->timestamp)
 #define GST_EVENT_SRC(event)		(GST_EVENT(event)->src)
 
+#define GST_EVENT_IS_INTERRUPT(event) (GST_EVENT_TYPE (event) == GST_EVENT_INTERRUPT)
+
 #define GST_SEEK_FORMAT_SHIFT	0
 #define GST_SEEK_METHOD_SHIFT	16
 #define GST_SEEK_FLAGS_SHIFT	20
@@ -77,9 +81,21 @@ typedef struct
   GstEventFlag 	flags;
 } GstEventMask;
 
-#define GST_EVENT_MASK_FUNCTION(functionname, a...)     \
+#ifdef G_HAVE_ISO_VARARGS
+#define GST_EVENT_MASK_FUNCTION(type,functionname, ...)      \
 static const GstEventMask*                              \
-functionname (GstPad *pad)                            	\
+functionname (type pad)                            	\
+{                                               	\
+  static const GstEventMask masks[] = {                 \
+    __VA_ARGS__,					\
+    { 0, }						\
+  };                                             	\
+  return masks;                                  	\
+}
+#elif defined(G_HAVE_GNUC_VARARGS)
+#define GST_EVENT_MASK_FUNCTION(type,functionname, a...)     \
+static const GstEventMask*                              \
+functionname (type pad)                            	\
 {                                               	\
   static const GstEventMask masks[] = {                 \
     a,							\
@@ -87,6 +103,7 @@ functionname (GstPad *pad)                            	\
   };                                             	\
   return masks;                                  	\
 }
+#endif
 
 /* seek events, extends GstEventFlag */
 typedef enum {
@@ -175,6 +192,8 @@ GstEvent*	gst_event_new	        	(GstEventType type);
 /* copy buffer */
 #define         gst_event_copy(ev)         	GST_EVENT (gst_data_copy (GST_DATA (ev)))
 
+gboolean 	gst_event_masks_contains 	(const GstEventMask *masks, GstEventMask *mask);
+
 /* seek event */
 GstEvent*	gst_event_new_seek		(GstSeekType type, gint64 offset);
 
