@@ -1,12 +1,12 @@
-#ifndef BISON_GRAMMAR_TAB_H
-# define BISON_GRAMMAR_TAB_H
+#ifndef BISON___GRAMMAR_TAB_H
+# define BISON___GRAMMAR_TAB_H
 
 #ifndef YYSTYPE
 typedef union {
     gchar *s;
     GValue *v;
     graph_t *g;
-    connection_t *c;
+    link_t *c;
     property_t *p;
     element_t *e;
 } yystype;
@@ -14,10 +14,10 @@ typedef union {
 # define YYSTYPE_IS_TRIVIAL 1
 #endif
 # define	IDENTIFIER	257
-# define	CONNECTION	258
-# define	BCONNECTION	259
-# define	FCONNECTION	260
+# define	LINK	258
+# define	BLINK	259
+# define	FLINK	260
 # define	VALUE	261
 
 
-#endif /* not BISON_GRAMMAR_TAB_H */
+#endif /* not BISON___GRAMMAR_TAB_H */
